import requests
import re
import logging
from .log import log
from .gitea import *

def get_codeowners(repository, branch):
    if not repository.startswith('https://'):
        return {}
    url = repository.replace('.git', f'/raw/branch/{branch}/CODEOWNERS')
    log.debug(f'Constructed CODEOWNERS URL: {url}')
    response = requests.get(url)
    log.debug(f'CODEOWNERS returned with HTTP {response.status_code}')
    if response.status_code != 200:
        log.error('Failed to query CODEOWNERS')
        return {}
    entries = response.text.splitlines()
    codeowners = {}
    for entry in entries:
        es = entry.split()
        codeowners[es[0]] = es[1:]
    log.debug(f'Returning CODEOWNERS: {codeowners}')
    return codeowners

def iterate_codeowners(changes, repository, branch):
    codeowners = get_codeowners(repository, branch)
    needed_codeowners = []
    for path in changes:
        q = query_codeowners(codeowners, path)
        if q:
            needed_codeowners.append(q)
    return needed_codeowners

def query_codeowners(codeowners, file):
    for pattern, owners in codeowners.items():
        if re.match(pattern, file):
            log.debug(f'Path {file} matched pattern {pattern}, returning CODEOWNERS: {owners}')
            return owners
    return None

def handle_approval(data):
    reviewer = data.get('sender', {}).get('login')
    if reviewer == 'butler':
        log.debug('Not serving myself.')
        return
    pr_data = data.get('pull_request', {})
    requester = pr_data.get('user', {}).get('login')
    if reviewer == requester:
        log.info('Refusing to handle approval from requester')
        return
    repo_data = pr_data.get('base', {}).get('repo', {})
    repo_name = repo_data.get('name')
    repo_branch = repo_data.get('default_branch')
    repo_url = repo_data.get('clone_url')
    pr_base = repo_data.get('full_name')
    pr_id = pr_data.get('number')
    if None in [pr_base, pr_id]:
        log.error('Insufficient data to handle approval')
        return
    changes = get_changes(pr_base, pr_id)
    # list containing lists of CODEOWNERS of the changed files
    all_owners = iterate_codeowners(changes, repo_url, repo_branch)
    good_approvals = 0
    # require one approval per set of CODEOWNERS
    needed_approvals = len(all_owners)
    for owners in all_owners:
        if reviewer in [owner.lstrip('@') for owner in owners]:
            good_approvals+=1
    info = f'merging {pr_id} in {pr_base}'
    if good_approvals == needed_approvals:
        log.info(f'Sufficient approvals, {info}')
        if not reviewed(pr_base, pr_id):
            review(pr_base, pr_id)
        comment(pr_base, pr_id, 'Alright, let me merge this for you.')
        merge(pr_base, pr_id)
    else:
        log.info(f'Not {info}, as it requires {needed_approvals} approvals')
